/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.objectweb.petals.tools.rmi.server.remote.implementations;

import java.net.URI;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Map;

import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.RobustInOnly;
import javax.xml.namespace.QName;

import org.objectweb.petals.tools.rmi.server.remote.interfaces.RemoteMessageExchangeFactory;
import org.objectweb.petals.tools.rmi.server.util.Convert;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfOperation.MEPPatternConstants;

/**
 * the RMI Message Exchange Factory.
 * @author Nicolas Salatge - EBM WebSourcing
 */
public class RemoteMessageExchangeFactoryImpl
extends UnicastRemoteObject implements RemoteMessageExchangeFactory {


    /**
     * the serialVersionUID.
     */
    private static final long serialVersionUID = -7802199904640493500L;

    /**
     * the concrete message exchange factory.
     */
    private MessageExchangeFactory concreteMef;


	/**
	 * HasMap containing all the messages exchanges.
	 */
	private Map<String, MessageExchange> messages;

    /**
     * the constructor.
     * @param msgs
	 *         the messages
     * @param concreteMef
     *         the concrete message exchange factory
     * @throws RemoteException
     *         impossible to realize a remote access
     */
    public RemoteMessageExchangeFactoryImpl(final Map<String, MessageExchange> msgs,
            final MessageExchangeFactory concreteMef) throws RemoteException {
        super();
        this.concreteMef = concreteMef;
        this.messages = msgs;
    }


    /**
     * TODO find an endpoint for this parameteters. Return a
     * <code>MessageExchangeDecorator</code> for Consumer
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createExchange(javax.xml.namespace.QName,
     *      javax.xml.namespace.QName)
     * @param serviceName
     *          the service name
     * @param operationName
     *          the operationName
     * @return
     *        the message exchange
     * @throws MessagingException
     *        impossible to create the message exchange
     */
    public MessageExchange createExchange(
            final QName serviceName, final QName operationName)
    throws MessagingException {
        // Create the concrete message exchange
        MessageExchange meC = this.concreteMef.createExchange(serviceName, operationName);


        // Convert into fake message exchange
        MessageExchange meF = Convert.
        convertConcreteMessageExchangeToFakeMessageExchange(meC, null);
        ((org.objectweb.petals.tools.rmi.common.serializable.MessageExchange) meF).setRole(meC.getRole());

        // Store the messages
        this.messages.put(meF.getExchangeId(), meC);

        return meF;
    }

    /**
     * Return a <code>MessageExchangeDecorator</code> for Consumer, as
     * <code>InOnlyExchange, InOutExchange,...</code>
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createExchange(java.net.URI)
     * @param pattern
     *          the pattern
     * @return
     *        the message exchange
     * @throws MessagingException
     *        impossible to create the message exchange
     */
    public MessageExchange createExchange(final URI pattern)
    throws MessagingException {
        // Create the concrete message exchange
        MessageExchange meC = this.concreteMef.createExchange(pattern);


        // Convert into fake message exchange
        MessageExchange meF = Convert.
        convertConcreteMessageExchangeToFakeMessageExchange(meC, null);
        ((org.objectweb.petals.tools.rmi.common.serializable.MessageExchange) meF).setRole(meC.getRole());

        // Store the messages
        this.messages.put(meF.getExchangeId(), meC);

        return meF;
    }


    /**
     * Return a <code>MessageExchangeDecorator</code> for Consumer, as
     * <code>InOnly</code>.
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createInOnlyExchange()
     * @return
     *        the message exchange
     * @throws MessagingException
     *        impossible to create the message exchange
     */
    public InOnly createInOnlyExchange() throws MessagingException {
        // Create the concrete message exchange
        MessageExchange meC = this.concreteMef.createInOnlyExchange();

        // Convert into fake message exchange
        InOnly meF = (InOnly) Convert.convertConcreteMessageExchangeToFakeMessageExchange(meC,
                MEPPatternConstants.IN_ONLY.value());
        ((org.objectweb.petals.tools.rmi.common.serializable.MessageExchange) meF).setRole(meC.getRole());

        // Store the messages
        this.messages.put(meF.getExchangeId(), meC);

        return meF;
    }

    /**
     * Return a <code>MessageExchangeDecorator</code> for Consumer, as
     * <code>InOptionalOut</code>.
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createInOptionalOutExchange()
     * @return
     *        the message exchange
     * @throws MessagingException
     *        impossible to create the message exchange
     */
    public InOptionalOut createInOptionalOutExchange()
    throws MessagingException {
        // Create the concrete message exchange
        MessageExchange meC = this.concreteMef.createInOptionalOutExchange();


        // Convert into fake message exchange
        InOptionalOut meF = (InOptionalOut) Convert.convertConcreteMessageExchangeToFakeMessageExchange(meC,
                MEPPatternConstants.IN_OPTIONAL_OUT.value());
        ((org.objectweb.petals.tools.rmi.common.serializable.MessageExchange) meF).setRole(meC.getRole());

        // Store the messages
        this.messages.put(meF.getExchangeId(), meC);

        return meF;
    }

    /**
     * Return a <code>MessageExchangeDecorator</code> for Consumer, as
     * <code>InOut</code>.
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createInOutExchange()
     * @return
     *        the message exchange
     * @throws MessagingException
     *        impossible to create the message exchange
     */
    public InOut createInOutExchange() throws MessagingException {
        // Create the concrete message exchange
        MessageExchange meC = this.concreteMef.createInOutExchange();

        // Convert into fake message exchange
        InOut meF = (InOut) Convert.convertConcreteMessageExchangeToFakeMessageExchange(meC,
                MEPPatternConstants.IN_OUT.value());
        ((org.objectweb.petals.tools.rmi.common.serializable.MessageExchange) meF).setRole(meC.getRole());

        // Store the messages
        this.messages.put(meF.getExchangeId(), meC);

        return meF;
    }

    /**
     * Return a <code>MessageExchangeDecorator</code> for Consumer, as
     * <code>RobustInOnly</code>.
     *
     * @see javax.jbi.messaging.MessageExchangeFactory#createRobustInOnlyExchange()
     * @return
     *        the message exchange
     * @throws MessagingException
     *        impossible to create the message exchange
     */
    public RobustInOnly createRobustInOnlyExchange() throws MessagingException {
        // Create the concrete message exchange
        MessageExchange meC = this.concreteMef.createRobustInOnlyExchange();

        // Convert into fake message exchange
        RobustInOnly meF = (RobustInOnly) Convert.convertConcreteMessageExchangeToFakeMessageExchange(meC,
                MEPPatternConstants.ROBUST_IN_ONLY.value());
        ((org.objectweb.petals.tools.rmi.common.serializable.MessageExchange) meF).setRole(meC.getRole());

        // Store the messages
        this.messages.put(meF.getExchangeId(), meC);

        return meF;
    }

}
